from _typeshed import Incomplete
from amsdal_data.connections.errors import AmsdalConnectionError as AmsdalConnectionError
from amsdal_data.connections.historical_base import HistoricalConnectionBase as HistoricalConnectionBase
from amsdal_data.table_schemas.base import TableSchemaServiceBase as TableSchemaServiceBase
from amsdal_data.table_schemas.constants import PRIMARY_PARTITION_KEY as PRIMARY_PARTITION_KEY, SECONDARY_PARTITION_KEY as SECONDARY_PARTITION_KEY
from amsdal_data.table_schemas.data_models.iceberg_table_column import ComplexType as ComplexType, IcebergDataTypes as IcebergDataTypes, IcebergTableColumn as IcebergTableColumn, ListType as ListType, MapType as MapType, StructType as StructType
from amsdal_utils.models.data_models.address import Address
from amsdal_utils.models.data_models.table_schema import TableColumnSchema, TableSchema
from amsdal_utils.models.enums import Versions
from amsdal_utils.query.data_models.filter import Filter as Filter
from amsdal_utils.query.data_models.order_by import OrderBy as OrderBy
from amsdal_utils.query.data_models.paginator import CursorPaginator, NumberPaginator
from amsdal_utils.query.data_models.query_specifier import QuerySpecifier as QuerySpecifier
from amsdal_utils.query.enums import Lookup
from amsdal_utils.query.utils import Q
from collections.abc import Callable as Callable
from pydantic import BaseModel
from pyspark.sql import DataFrame as DataFrame, Row, SparkSession
from typing import Any, ClassVar

SUPPORTED_NESTED_FIELDS: Incomplete
logger: Incomplete
DEFAULT_PACKAGES: Incomplete
DEFAULT_CONNECTION_CONFIGS: Incomplete

def random_string(length: int = ...) -> str: ...

class IcebergTableColumnSchema(TableColumnSchema):
    type: type | ComplexType

address_struct: Incomplete
reference_struct: Incomplete

class SqlOperatorTemplate(BaseModel):
    build_statement: ClassVar[Callable[[Any, str, str, Any], str]]
    build_value: ClassVar[Callable[[Any, Any], Any]]
    template: str | Callable[[Any], str]
    value_template: str | None
    value_modifier: Callable[[Any], Any] | None
    def build_statement(self, field_name: str, filter_key: str, value: Any) -> str: ...
    def build_value(self, value: Any) -> Any: ...

sql_operator_map: dict[Lookup, SqlOperatorTemplate]

class IcebergHistoricalConnection(HistoricalConnectionBase):
    _connection: Incomplete
    _catalog_name: Incomplete
    _namespace_name: Incomplete
    _branches_lineage: Incomplete
    _branches_changes: Incomplete
    _branches_revert: Incomplete
    _filter_counter: int
    def __init__(self, catalog_name: str = ..., namespace: str = ...) -> None: ...
    def _build_columns_from_schema(self, table_schema: TableSchema) -> list[IcebergTableColumn]: ...
    @staticmethod
    def _to_sql_type(type_: Any) -> IcebergDataTypes | ComplexType: ...
    @property
    def table_schema_manager(self) -> TableSchemaServiceBase: ...
    @property
    def connection(self) -> SparkSession: ...
    def connect(self, app_name: str = ..., catalog_name: str = ..., namespace_name: str = ..., master_node: str = ..., log_level: str = ..., configs: dict[str, str] | None = ..., **kwargs: Any) -> None:
        """
        Connects to the PySpark cluster. Raises an AmsdalConnectionError if the connection is already established.
        :param app_name: the name of the application
        :type app_name: str
        :param configs: the connection configs parameters to pass to the SparkSession
        :type configs: dict[str, str] | None
        :param kwargs: the connection parameters
        :type kwargs: Any

        :return: None
        """
    def disconnect(self) -> None: ...
    def _current_branch_name(self) -> str | None: ...
    def _set_branch(self, branch: str) -> None: ...
    def begin(self) -> None: ...
    def commit(self) -> None: ...
    def revert(self) -> None: ...
    def on_transaction_complete(self) -> None: ...
    def rollback(self) -> None: ...
    def _preprocess_data(self, value: Any, field_type: str | ComplexType) -> tuple[str, Any]: ...
    def put(self, address: Address, data: dict[str, Any]) -> None: ...
    def _process_table_description_type(self, data_type: str) -> str | ComplexType: ...
    def _get_table_description(self, table_name: str) -> list[IcebergTableColumn]: ...
    def _get_latest_snapshot_id(self, table_name: str) -> int | None: ...
    def _execute(self, stm: str, values: dict[str, Any] | None = ...) -> DataFrame: ...
    def _get_table_name(self, address: Address) -> str: ...
    def _get_full_table_name(self, address: Address) -> str: ...
    def _table_name_with_catalog(self, table_name: str) -> str: ...
    def _get_metadata_fields(self, join_table: str = ..., fields_rename_prefix: str = ..., *, needs_rename: bool = ..., with_references: bool = ...) -> str: ...
    def _concat_references(self, field_prefix: str, rename_field: str) -> str: ...
    def _metadata_next_version(self, metadata_table_name: str) -> str: ...
    def __select_statement(self, select_only: str, table_name: str, main_table_name: str, metadata_join_table: str, conditions: Q | None = ..., *, is_internal_table: bool) -> tuple[str, dict[str, Any]]: ...
    def _build_select_statement(self, address: Address, query_specifier: QuerySpecifier | None = ..., conditions: Q | None = ..., pagination: NumberPaginator | CursorPaginator | None = ..., order_by: list[OrderBy] | None = ...) -> tuple[str, dict[str, Any]]: ...
    def _build_count_statement(self, address: Address, conditions: Q | None = ...) -> tuple[str, dict[str, Any]]: ...
    def _process_row_value(self, value: Any) -> Any: ...
    def _process_row_into_result(self, row: Row, table_description: dict[str, IcebergTableColumn]) -> dict[str, Any]: ...
    def query(self, address: Address, query_specifier: QuerySpecifier | None = ..., conditions: Q | None = ..., pagination: NumberPaginator | CursorPaginator | None = ..., order_by: list[OrderBy] | None = ...) -> list[dict[str, Any]]: ...
    def _query(self, address: Address, class_version: str | Versions, query_specifier: QuerySpecifier | None = ..., query: Q | None = ..., pagination: NumberPaginator | CursorPaginator | None = ..., order_by: list[OrderBy] | None = ...) -> list[dict[str, Any]]: ...
    def count(self, address: Address, conditions: Q | None = ...) -> int: ...
    def _get_conditions_statement(self, conditions: Q, *, is_grouped: bool = ...) -> tuple[str, dict[str, Any]]: ...
    def _get_filter_statement(self, filter_item: Filter) -> tuple[str, dict[Any, Any]] | tuple[None, None]: ...
    @staticmethod
    def _to_sql_value(value: Any) -> Any: ...
    def prepare_connection(self) -> None: ...
